//
//  editor.js
//  DGWebEditor
//
//  Created by Roy on 2019/12/2.
//  Updated by Detail on 2020/4/26.
//  Copyright © 2019 dgtle.com. All rights reserved.
//
let BlockEmbed = Quill.import('blots/block/embed');
let Inline = Quill.import("blots/inline");
class DividerBlot extends BlockEmbed {}
DividerBlot.blotName = 'divider';
DividerBlot.tagName = 'hr';

class ImageBlot extends BlockEmbed {

    static create(value) {
        let figure = super.create();
        figure.setAttribute('contenteditable', false);

        let imgContainer = document.createElement('div');
        imgContainer.className = "img-container";
        let img = document.createElement('img');
        img.onload = function () {
            if (this.src.indexOf('blob') === 0) {
                URL.revokeObjectURL(this.src);
            }
        };
        if (typeof value === 'string') {
            img.setAttribute('src', value);
        } else {
            img.setAttribute('src', value.url || '');
        }
        if (typeof value.id === 'string') {
            img.setAttribute('id', value.id);
        }
        if (typeof value.alt === 'string') {
            img.setAttribute('alt', value.alt);
        }

        // 如果有选择文件，把文件对象绑定在 img 标签上，上传用
        if (value.file) {
            $.data(img, 'file', value.file);
        }

        imgContainer.appendChild(img);
        figure.appendChild(imgContainer);

        let figcaption = document.createElement('figcaption');
        figcaption.innerText = value.alt || '';
        figcaption.innerText = '';
        figure.appendChild(figcaption);

        return figure;
    }

    static formats(figure) {
        let img = figure.querySelector('img');
        return {
            id: img ? (img.getAttribute('id') || '') : '',
            url: img ? (img.getAttribute('src') || '') : '',
            alt: img ? (img.getAttribute('alt') || '') : ''
        };
    }

    static value(figure) {
        let img = figure.querySelector('img');
        return {
            id: img ? (img.getAttribute('id') || '') : '',
            url: img ? (img.getAttribute('src') || '') : '',
            alt: img ? (img.getAttribute('alt') || '') : ''
        };
    }

    format(name, value) {
        let img = this.domNode.querySelector('img');
        let figcaption = this.domNode.querySelector('figcaption');

        if (name === 'alt') {
            img.setAttribute(name, value);
            figcaption.innerText = value;
        } else if (name === 'src') {
            img.setAttribute(name, value);
        } else if (name === 'id') {
            img.setAttribute(name, value);
        } else {
            super.format(name, value);
        }
    }

    // 构造
    constructor(domNode) {
        super(domNode);
        let _this = this;
        let $domNode = $(domNode);

        // 图片容器
        _this.$imgContainer = $domNode.find(".img-container");

        // 图片
        _this.$img = $domNode.find("img");

        // 图片描述
        _this.$figcaption = $domNode.find("figcaption");

        // 关闭按钮
        _this.$closeButton = (function () {
            let $button = $('<span class="close-button"/>');
            $button.click(function () {
                let index = quill.getIndex(_this);
                _this.remove();
                quill.focus();
                quill.setSelection(index);
            });
            _this.$imgContainer.append($button);
            return $button;
        })();

        // 编辑描述按钮
        _this.$editDescButton = (function () {
            let $button = $('<span class="edit-desc-button"><i/></span>');
            $button.click(function () {
                let message = {
                    title: "添加图片描述",
                    placeholder: '不超过120字',
                    maxLength: 120,
                    type:1
                };
                let desc = prompt(JSON.stringify(message), _this.$figcaption.text());
                if (desc != null) {
                    desc = $.trim(desc);
                    if (desc.length > 120) {
                        desc = desc.substring(0, 120)
                    }
                    _this.setImageDesc(desc);
                }
                quill.focus();
                quill.setSelection(quill.getIndex(_this) + 1);
            });
            _this.$imgContainer.append($button);
            return $button;
        })();

        // 状态视图提示信息
        _this.$statusInfoLabel = $('<label>图片上传失败</label>');
        // 重试按钮
        _this.$statusRetryButton = (function () {
            let $button = $('<span class="retry-button"><i/><b>重试</b></span>');
            $button.click(function () {
                _this.beginUploadImageIfNeed();
            });
            return $button;
        })();
        // 状态视图
        _this.$statusView = (function () {
            let $view = $('<div class="status-view"/>');
            let $content = $('<span class="content"/>');
            $content.append(_this.$statusInfoLabel);
            $content.append(_this.$statusRetryButton);
            $view.append($content);
            _this.$imgContainer.append($view);
            return $view;
        })();

        _this.updateUI(0);
        _this.beginUploadImageIfNeed();

    }

    updateUI(status, data) {
        let _this = this;

        _this.status = status;

        switch (status) {
            case 0: // 初始化状态
                // data = null
                _this.$statusView.hide();
                break;
            case 1: // 上传中
                // data = Percent
                let uploadPercent = data + '%';
                _this.$statusInfoLabel.text('图片上传中 ' + uploadPercent);
                _this.$statusInfoLabel.show();
                _this.$statusRetryButton.hide();
                _this.$statusView.show();
                break;
            case 2: // 上传成功
                // data = APIData
                _this.$statusView.hide();
                break;
            case 3: // 上传失败
                // data = Error or APIData
                _this.$statusInfoLabel.text('图片上传失败');
                _this.$statusInfoLabel.show();
                _this.$statusRetryButton.show();
                _this.$statusView.show();
                break;
            default:
                break;
        }
    }

    beginUploadImageIfNeed() {
        let _this = this;
        if (_this.uploadRequest) {
            _this.uploadRequest.abort();
        }

        let imageURL = _this.$img.attr('src');

        // http: https: data:
        let block = imageURL.split(':');
        let scheme = block[0];
        if (scheme !== 'blob') {
            return
        }

        _this.updateUI(1, 0);

        let file = _this.$img.data('file');
        _this.uploadRequest = uploadPicture({
            file: file,
            watermark: true,
            progress: function (percent) {
                _this.updateUI(1, percent);
            },
            success: function (data) {
                try {
                    if (data.result.id) {

                        let oldDelta = quill.editor.delta;

                        // 上传成功，更新 id 和 path
                        _this.$img.attr('id', data.result.id);
                        // 这里服务端会返回不含域名的图片路径，这里补充完整
                        _this.$img.attr('data-path', "http://s1.dgtle.com" + data.result.img_url);

                        // 上传图片成功，发送通知
                        native.cell({
                            __action: 'editorOnEvent',
                            event: 'text-change',
                            source: 'api',
                            value: {
                                delta: quill.editor.delta,
                                oldDelta: oldDelta
                            }
                        });

                        _this.updateUI(2, data);
                    } else {
                        _this.updateUI(3, data);
                    }
                } catch (err) {
                    _this.updateUI(3, err);
                }
            },
            error: function (err) {
                _this.updateUI(3, err);
            }
        });
    }

    setImageDesc(desc) {
        let _this = this;

        _this.$figcaption.text(desc);
        _this.$img.attr('alt', desc);

        if (desc != null && desc !== '') {
            _this.$editDescButton.find('i').addClass("edit");
        } else {
            _this.$editDescButton.find('i').removeClass("edit");
        }


    }

    // 释放
    detach() {
        if (this.uploadRequest) {
            this.uploadRequest.abort();
        }
        super.detach();
    }

}
ImageBlot.blotName = 'image';
ImageBlot.tagName = 'figure';
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

// 新增
function geturl (href){
    let http = href.split('//')[0]
    let host = href.split('//')[1].split('/')[0];
    return http+'//'+host
  }
let bgImg = 'http://s1.dgtle.com/dgtle_img/wap/net_normal@2x.png.png'//链接默认背景图
let LinkDom;//链接卡片节点
  //卡片样式链接
  class LinkBlot extends BlockEmbed {
    static create(url) {
        let node = super.create();
        node.setAttribute("contenteditable", false);
        url.href?node.setAttribute("href", url.href):node.setAttribute("href", url);
        url.class?node.setAttribute("class", url.class):'';
        if(url.class){
          let div = document.createElement("div");
          div.className = 'link-card-wrap';
          div.innerHTML=popUp(3,url.text,geturl(url.href),url.background);
          node.appendChild(div)
        }
  
      return node;
    }
  
    static formats(domNode) {
      let background = $(domNode).find('.link-card-background').css('background-image');
      let url = (background && background.indexOf('file:///')==-1 )? background.split('("')[1].split('")')[0]:bgImg;
      let text = $(domNode).find('.link-card-content').text()||''
      return {
        href: domNode.getAttribute("href") || '',
        class: domNode.getAttribute("class") || '',
        text: text,
        host:$(domNode).find('.link-card-href').text()||'',
        background:url
      }
    }
    static value(domNode) {
      let background = $(domNode).find('.link-card-background').css('background-image');
      let url =(background && background.indexOf('file:///')==-1 ) ? background.split('("')[1].split('")')[0]:bgImg;
      let text = $(domNode).find('.link-card-content').text()||''
       return {
         href: domNode.getAttribute("href") || '',
         class: domNode.getAttribute("class") || '',
         text:text,
         host:$(domNode).find('.link-card-href').text()||'',
         background:url
       }
     }
  
    format(name, value) {
      if (name === "link" && value) {
        this.domNode.setAttribute("href", value);
      } else {
        super.format(name, value);
      }
    }
  
    formats() {
      let formats = super.formats();
      return formats;
    }
  
    constructor(domNode) {
      super(domNode);
      let _this = this;
      let type;
      let $domNode = $(domNode);
      
    
      $domNode.click(function (e) {
        //组织默认行为
              e = e.preventDefault ? e.preventDefault() : e.returnValue = false;
        //阻止冒泡
          if (e && e.stopPropagation) {
            e.stopPropagation();
          } else {
            window.event.cancelBubble = true;
          }
          quill.setSelection(quill.getIndex(_this)+1);
          LinkDom = $(this)
          type = $(this).hasClass('link-card') ? 2 : 1;
          let href = $(this).attr('href');
          let content = $(this).find('.link-card-content').text()||$(this).text();
          LinkBubble($(this),type,href,content);
      })
        // 关闭按钮
        _this.$Container = $domNode.find(".link-content");
        _this.$closeButton = (function () {
        let $button = $('<span class="close-button"/>');
        $button.click(function (e) {
            stopPropagation(e)
            let index = quill.getIndex(_this);
            _this.remove();
            $('.LinkBubble-content').remove();
        });
        _this.$Container.append($button);
        return $button;
        })();



    }
  }
  LinkBlot.blotName = "LinkBlot";
  LinkBlot.tagName = "div";
  LinkBlot.className = "link-card";
  
  //普通样式链接
  class Link extends Inline {
    static create(value) {
      let node =super.create();
    //   console.log(value)
      if (value && !value.href) {
        // 适应原本的Link Blot
        node = super.create(value);
        node.setAttribute('href', value);
        node.setAttribute('target', '_blank');
        node.setAttribute('title', node.textContent);
      } else if(!value.background) {
        // 自定义Link Blot
        node = super.create(value.href);
        node.innerText = value.text;
        node.href = value.href;
      }else{
        node.setAttribute('href', value.href);
      }
      return node;
    }
  
    static formats(domNode) {
        // console.log(domNode)
      return  domNode.getAttribute('href') || true;
    }
  
    format(name, value) {
        // console.log(name)
      if (name === 'link' && value) {
        this.domNode.setAttribute('href', value);
      } else {
        super.format(name, value);
      }
    }
  
    formats() {
      let formats = super.formats();
      // formats['link'] = LinkBlot.formats(this.domNode);
      return formats;
    }
  
    constructor(domNode) {
      super(domNode);
      let _this = this;
      let type;
      let $domNode = $(domNode);
      

      $domNode.click(function (e) {
        stopPropagation(e)
        LinkDom = $(this)
        let href = $(this).attr('href');
        let content = $(this).text();
        // LinkBubble($(this),1,href,content);
      });
       
    }
  
  }
  Link.blotName = 'link';
  Link.tagName = 'a';

  //处理视频
class VideoBlot extends BlockEmbed {
    static create(url) {
      let node = super.create();
      node.setAttribute("src", url);
      node.setAttribute("frameborder", "0");
      node.setAttribute("allowfullscreen", true);
      return node;
    }
  
    static formats(node) {
      let format = {};
      if (node.hasAttribute("height")) {
        format.height = node.getAttribute("height");
      }
      if (node.hasAttribute("width")) {
        format.width = node.getAttribute("width");
      }
      return format;
    }
  
    static value(node) {
      return node.getAttribute("src");
    }
  
    format(name, value) {
      if (name === "height" || name === "width") {
        if (value) {
          this.domNode.setAttribute(name, value);
        } else {
          this.domNode.removeAttribute(name, value);
        }
      } else {
        super.format(name, value);
      }
    }
  }
  VideoBlot.blotName = "video";
  VideoBlot.tagName = "iframe";

  //导购代码
class DividerBuy extends BlockEmbed {
    static create(value) {
      let node = super.create();
      node.setAttribute("contenteditable", false);
      node.innerHTML = ` <div class="buy-main">
          <div class="buy-text">
          <span class="pic-box" style="background-image:url(${value.img});">
              </span>
              <div class="main-concent">
                  <span class="h-title">${value.title}</span>
                  <span class="intro">${value.intro}</span>
              </div>
          </div>
          <span class="clearfix">
              <span class="price ellipsis">${value.price}<em>元</em></span>
              <a target="_blank" href="${value.siteUrl}" class="go-site"><span>官方网站</span></a>
            <a target="_blank" href="${value.buyUrl}" class="go-buy"><span>立即购买</span></a>
          </span>
      </div> `;
      return node;
    }
  
    static value(domNode) {
      let dom = $(domNode);
      let title = dom.find(".h-title").text();
      let intro = dom.find(".intro").text();
      let price = dom
        .find(".price")
        .text()
        .replace("¥", "");
      let siteUrl = dom.find(".go-site").attr("href");
      let buyUrl = dom.find(".go-buy").attr("href");
      let url =
        dom.find(".pic-box").css("background-image") != "none" ?
        dom
        .find(".pic-box")
        .css("background-image")
        .split('("')[1]
        .split('")')[0] :
        "";
      let img = dom.find("img").attr("src") ? dom.find("img").attr("src") : "";
      return {
        title: dom ? title : "",
        intro: dom ? intro : "",
        price: dom ? parseFloat(price) : "",
        siteUrl: dom ? siteUrl : "",
        buyUrl: dom ? buyUrl : "",
        img: dom ? url || img : ""
      };
    }
  }
  DividerBuy.blotName = "dividerBuy";
  DividerBuy.tagName = "div";
  DividerBuy.className = "buy-wrap";
  
////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


Quill.register(DividerBlot);
Quill.register(ImageBlot);
Quill.register(LinkBlot);
Quill.register(Link);
Quill.register(VideoBlot);
Quill.register(DividerBuy);

let quill = new Quill('#editor-container', {
    placeholder: '正文'
});

quill.root.setAttribute('spellcheck', 'false');
quill.root.setAttribute('autocomplete', 'off');
quill.root.setAttribute('autocorrect', 'off');
quill.root.setAttribute('autocapitalize', 'none');

quill.ext = {
    getPlaceholder: function () {//
        return document.getElementById("editor-container").firstChild.getAttribute('data-placeholder');
    },
    setPlaceholder: function (text) {
        document.getElementById("editor-container").firstChild.setAttribute('data-placeholder', text);
    },
    insertDivider: function () { //插入分割线
        let range = quill.getSelection();
        if (range == null) {
            return
        }

        let p = quill.getLine(range.index)[0].domNode;
        // 如果当前选中行是空白行则少插入一个换行符
        if (p.tagName.toLowerCase() === 'p' &&
            p.children.length === 1 &&
            p.children[0].tagName.toLowerCase() === 'br') {
            quill.insertEmbed(range.index, 'divider', true, Quill.sources.USER);
            quill.setSelection(range.index + 1, Quill.sources.SILENT);
        } else {
            quill.insertText(range.index, '\n', Quill.sources.USER);
            quill.insertEmbed(range.index + 1, 'divider', true, Quill.sources.USER);
            quill.setSelection(range.index + 2, Quill.sources.SILENT);
        }
    },
    getContent: function () { //获取内容
        // "coverURL": "blob:null/10511637-bb61-4376-91e8-3f381ed7d19f",
        // "coverURL": "data:image/png;base64,XXX...XXX",
        // "coverURL": "https://x.x/x.png",
        // "titleText": "标题",
        // "bodyHTML": "<h1>...</h1><figure><img id='666' src='https://x.x/x.png' alt='xxx'></figure>"

        let $coverContainer = $('#cover-container');

        let coverURL = $coverContainer.css('background-image')
            .replace('url("', '').replace('")', '');

        // 封面图如果不是 http 开头的则使用 path 代替
        if (coverURL.indexOf('http') !== 0) {
            coverURL = $coverContainer.attr('data-path');
        }

        let $content;
        // quill 官方定义，即使内容为空，取到的内容里也还是会有一个 '\n' 字符
        if (quill.getLength() === 1 && quill.getText() === '\n') {
            $content = $('<div></div>');
        } else {
            $content = $('<div>' + quill.root.innerHTML + '</div>');
            // $('.link-card').removeAttr('contenteditable');/
            $content.find('.link-card').each(function (i) {
                let innerHTML = $(this).html().replace(/&quot;/ig,'')
                $(this).html(innerHTML)
              })
            // 清理 figure 元素，保证内容为约定的格式
            $content.find('figure').each(function (i) {
                let $figure = $(this);
                $figure.removeAttr('contenteditable');

                let $img = $figure.find('img');

                // 删除没有 图片ID 也不是 http 开头的 img
                if (!$img.attr('id') && $img.attr('src').indexOf('http') !== 0) {
                    $figure.remove();
                    return
                }

                if ($img.attr('src').indexOf('http') !== 0) {
                    $img.removeAttr('src');
                }
                if ($img.attr('data-path')) {
                    $img.attr('src', $img.attr('data-path'));
                }
                $img.removeAttr('data-path');

                $figure.empty();
                $figure.append($img);

                // 删除增加图片时图片上方生成的空白换行
                let $prev = $figure.prev();
                if ($prev.length > 0) {
                    let p = $prev[0];
                    if (p.tagName.toLowerCase() === 'p') {
                        if (p.children.length === 1 && p.children[0].tagName.toLowerCase() === 'br') {
                            p.remove();
                        }
                    }
                }
            });
             // 删除增加链接上方生成的空白换行
            $content.find('.link-card').each(function (i) {
               let $card = $(this);
               let $prev = $card.next().next();
               if ($prev.length > 0) {
                   let p = $prev[0];
                   if (p.tagName.toLowerCase() === 'p') {
                       if (p.children.length === 1 && p.children[0].tagName.toLowerCase() === 'br') {
                           p.remove();
                       }
                   }
               }
            });
        }
        return {
            coverURL: coverURL,
            titleText: $('#title-editor').text().trim(),
            bodyHTML: $content.html().replace(/<p><\/p>/g, "")
        };
      
    },
    setContent: function (obj) {//设置内容
        if (obj.coverURL && obj.coverURL !== "") {
            coverImageManager.updateUI(1, obj.coverURL);
            coverImageManager.updateUI(3);
        } else {
            coverImageManager.updateUI(0);
        }

        $('#title-editor').text(obj.titleText);
        $('#title-tip').text('');

        quill.setContents([]);

        let $content = $('<div>' + obj.bodyHTML + '</div>');

        // 给所有前一个元素还是图片的图片元素前加入一个换行符，用来让用户能在图片之间输入文字，获取内容时会将空白换行移除
        $content.find('figure').each(function (i) {
            let $figure = $(this);
            let $prev = $figure.prev();
            if ($prev.length > 0) {
                if ($prev[0].tagName.toLowerCase() === 'figure'||$prev[0].className === 'link-card') {
                    $figure.before('<br>');
                }
            }
        });
        $content.find('.link-card').each(function (i) {
            let $figure = $(this);
            let $prev = $figure.prev();
            if ($prev.length > 0) {
                if ($prev[0].className === 'link-card') {
                    $figure.before('<br>');
                }
            }
        });

        // 这里 <p><br><\/p> 会被解析成 两个 <p><br><\/p>，提前过滤
        let html = $content.html().replace(/<p><br><\/p>/g, "<br>");
        quill.pasteHTML(html);
    },
    getFocusInputName: function () {
        if ($('#title-editor').is(':focus')) {
            return 'title';
        } else if (quill.hasFocus()) {
            return 'body';
        } else {
            return null;
        }
    },
    insertImage: function () {
        let _this = this;
        choosePicture({
            multiple: true,
            onChange: function (files) {
                if (files.length > 9) {
                    alert('一次最多仅可选择 9 张图片，请重新选择');
                    return
                }
                for (let i = 0; i < files.length; i++) {
                    let file = files[i];

                    // blob:null/10511637-bb61-4376-91e8-3f381ed7d19f
                    let url = URL.createObjectURL(file);
                    _this._insertImage(url, '', file);

                    // data:image/png;base64,XXX...XXX
                    // let reader = new FileReader();
                    // reader.onload = function (e) {
                    //     let url = e.target.result
                    //     _this._insertImage(url, '', file);
                    // };
                    // reader.readAsDataURL(file);
                }
            }
        });
    },
    _insertImage: function (url, alt, file) {
        let range = quill.getSelection(true);
        quill.insertText(range.index, '\n', Quill.sources.USER);
        quill.insertEmbed(range.index + 1, 'image', {
            url: url,
            alt: alt,
            file: file
        }, Quill.sources.USER);
        quill.setSelection(range.index + 2, Quill.sources.SILENT);
    },
	//格式化为加粗
	formatTextBold : function  (value) {
	    let format = quill.getFormat();
	    quill.format('bold',value,'user')
	},
	//获取加粗的状态
	getBoldStatus : function  () {
	    let format = quill.getFormat();
	    native.cell({
	                __action: 'editorOnEvent',
	                event: 'bold-change',
	                source: 'user',
	                value: {
	                    bold: format.bold == true
	                }
	            });
	},
    setLink: function  (){ //设置链接
        let range = quill.getSelection(true);
        let text = quill.getText(range.index, range.length);
        if(text){
            let arr = quill.getContents(range.index, range.length).ops
            let attributes1 = arr[0].attributes
            let attributes2 = arr[arr.length - 1].attributes
            let href = '';
            if (attributes1 && attributes1.link) {
              href = attributes1.link
            } else if (attributes2 && attributes2.link) {
              href = attributes2.link
            }
            let message = {
              title: "编辑链接",
              type: 2
            };
            let obj = {
                href:href,
                text:text
            };
            let desc = prompt(JSON.stringify(message), JSON.stringify(obj));
            if (desc == null) {
                return 
            }
            desc = JSON.parse(desc);
            quill.format("link", desc.href);
        }else {
            let message = {
              title: "插入链接",
              type: 2
            };
            let desc = prompt(JSON.stringify(message), '');
            if (desc == null) {
                return 
            }
            desc = JSON.parse(desc);
            quill.insertText(range.index, "\n", Quill.sources.USER);
            //value=对象{href:value,text:linkMsg}
            quill.insertEmbed(range.index + 1, "link", desc,  Quill.sources.USER);
            quill.setSelection(range.index + desc.text.length+1, Quill.sources.SILENT);
        }
      
    }
};

function choosePicture(options) {
    let fileInput = document.getElementById('image-input');
    if (!fileInput) {
        fileInput = document.createElement('input');
        fileInput.setAttribute('id', 'image-input');
    }
    fileInput.setAttribute('type', 'file');
    fileInput.setAttribute('accept', 'image/png, image/gif, image/jpeg, image/bmp, image/x-icon');
    fileInput.setAttribute('style', 'display: none;');
    document.body.appendChild(fileInput);

    if (options.multiple === true) {
        fileInput.setAttribute('multiple', 'multiple');
    } else {
        fileInput.removeAttribute('multiple')
    }
    fileInput.setAttribute('value', '');

    fileInput.onchange = function () {
        options.onChange(fileInput.files);
        fileInput.value = '';
        fileInput.remove();
    };

    fileInput.click();
}

function uploadPicture(options) {
    let formData = new FormData();
    let fileExtension = options.file.name.split('.')[1];
    formData.append("file", options.file, 'image.' + fileExtension);
    if (options.watermark === false) {
        formData.append("watermark", 0);
    }

    let uploadURL = sessionStorage.getItem("upload_url");
    let authorization = sessionStorage.getItem("authorization");

    return $.ajax({
        url: uploadURL,
        type: 'POST',
        data: formData,
        timeout: 120 * 1000,
        async: true,
        processData: false,
        contentType: false,
        cache: false,
        headers: {
            'Authorization': authorization
        },
        xhr: function () {
            let xhr = $.ajaxSettings.xhr();
            if (xhr.upload) {
                xhr.upload.addEventListener('progress', function (e) {
                    if (e.lengthComputable) {
                        let percent = Math.floor(e.loaded / e.total * 100 * 0.99);
                        // console.log('图片上传中', e, percent);
                        options.progress(percent);
                    }
                }, false);
            }
            return xhr;
        },
        success: function (data) {
            // console.log('图片上传完成', data);
            options.success(data)
        },
        error: function (err) {
            // console.log('图片上传失败', err);
            options.error(err)
        },
    });
}

quill.on('selection-change', function (range, oldRange, source) {
    if(Object.keys(quill.getFormat(range.index))=='link'){
        let href = quill.getFormat(range.index).link;
        let range2 = document.getSelection().getRangeAt(0)
        let nodeName = range2.commonAncestorContainer.parentElement;
        LinkDom = $(nodeName)
        LinkBubble($(nodeName),1,href,nodeName.innerHTML);
    }
    // else if(Object.keys(quill.getFormat(range.index+1))=='link'){
    //     // let href = quill.getFormat(range.index+1).link;
    //     // let range2 = document.getSelection().getRangeAt(0)
    //     // let nodeName = range2.commonAncestorContainer.nextSibling;
    //     // LinkDom = $(nodeName)
    //     // console.log(nodeName.innerHTML)
    //     // LinkBubble($(nodeName),1,href,nodeName.innerHTML);
    // }
    else{
        $('.LinkBubble-content').remove();
    }
   
    if (oldRange == null) {
        native.cell({
            __action: 'editorOnFocusChange',
            inputName: 'body'
        });
    }

    native.cell({
        __action: 'editorOnEvent',
        event: 'selection-change',
        source: source,
        value: {
            range: range,
            oldRange: oldRange
        }
    });
});

quill.on('text-change', function (delta, oldContents, source) {
    // $('.LinkBubble-content').remove();
    native.cell({
        __action: 'editorOnEvent',
        event: 'text-change',
        source: source,
        value: {
            delta: delta,
            oldContents: oldContents
        }
    });
});

$('#title-editor').on('paste', function (event) {
    // 清理粘贴的内容，只保留文本
    if (event.originalEvent.clipboardData) {
        // 阻止默认行为
        event.preventDefault();
        let clipboardData = event.originalEvent.clipboardData;
        // 获取剪贴板的文本
        let text = clipboardData.getData('text');
        if (window.getSelection && text !== '' && text !== null) {
            // 创建文本节点
            let textNode = document.createTextNode(text);
            // 在当前的光标处插入文本节点
            let range = window.getSelection().getRangeAt(0);
            // 删除选中文本
            range.deleteContents();
            // 插入文本
            range.insertNode(textNode);
			
			//移动光标到插入文本位置末尾
			range.setStart(textNode,textNode.length);
			range.setEnd(textNode,textNode.length)
			
			let _this = $(this);
			titleChange(_this);
        }
    }
});

$('#title-editor').on('focusin', function (event) {
    native.cell({
        __action: 'editorOnFocusChange',
        inputName: 'title'
    });
});

//获取文本字节长度,一个英文半个字符,一个汉字一个字符
function getCharLength(value){
     let n = value.trim().replace(/[^\x00-\xff]/g,"aa").length/2;
     return Math.ceil(n);
};

$('#title-editor').on('input', function (event) {
	let _this = $(this);
	titleChange(_this);
});

//标题发生改变
function titleChange(_this){
	let length = getCharLength(_this.text());
	let tip = $('#title-tip');
	if (length >= 41) {
	    tip.css('color', '#F58C8C');
	    tip.text('已超出 ' + (length - 40) + ' 个字');
	} else {
	    tip.css('color', '#C6C6C6');
	    if (length >= 30) {
	        tip.text('还可以输入 ' + (40 - length) + ' 个字');
	    } else {
	        tip.text('');
	    }
	}
	_this.children("div").remove();
	_this.children("br").remove();
	
	// 标题发生改变通知
	native.cell({
	    __action: 'titleOnTextChange'
	});
};

class CoverImageManager {

    constructor() {
        let _this = this;

        _this.$coverMask = $('#cover-container .mask');
        _this.$coverTipIcon = $('#cover-tip .icon');
        _this.$coverTipLabel = $('#cover-tip label');
        _this.$coverRetryButton = $('#cover-tip .retry-button');

        _this.$coverContainer = $('#cover-container');
        _this.$coverContainer.click(function (e) {
            _this.handleCoverClick();
        });

        _this.updateUI(0);
    }

    updateUI(status, data) {
        let _this = this;

        _this.status = status;

        switch (status) {
            case 0: // 初始化状态，没有选择图片
                // data = null
                _this.$coverContainer.attr('data-id', '');
                _this.$coverContainer.attr('data-path', '');

                _this.$coverContainer.css('background-image', 'none');
                _this.$coverMask.css('background-color', 'transparent');
                _this.$coverTipIcon.show();
                _this.$coverTipLabel.text('上传封面');
                _this.$coverRetryButton.hide();
                break;
            case 1: // 选择了图片
                // data = image url
                _this.coverURL = data;
                _this.$coverContainer.css('background-image', 'url(' + data + ')');
                _this.$coverMask.css('background-color', 'rgba(0,0,0,0.49)');
                break;
            case 2: // 上传中
                // data = Percent
                _this.$coverTipIcon.hide();
                let uploadPercent = data + '%';
                _this.$coverTipLabel.text('图片上传中 ' + uploadPercent);
                _this.$coverRetryButton.find('i').hide();
                _this.$coverRetryButton.find('b').text('取消上传');
                _this.$coverRetryButton.show();
                break;
            case 3: // 上传成功
                // data = APIData

                // 更新 id 和 path
                if (data) {
                    _this.$coverContainer.attr('data-id', data.result.id);
                    _this.$coverContainer.attr('data-path', data.result.img_url);
                }

                _this.$coverTipIcon.show();
                _this.$coverTipLabel.text('更换图片');
                _this.$coverRetryButton.hide();
                break;
            case 4: // 上传失败
                // data = Error or APIData
                _this.$coverTipIcon.hide();
                _this.$coverTipLabel.text('图片上传失败');
                _this.$coverRetryButton.find('i').show();
                _this.$coverRetryButton.find('b').text('重新选择');
                _this.$coverRetryButton.show();
                break;
            default:
                break;
        }
    }

    handleCoverClick() {
        let _this = this;

        switch (_this.status) {
            case 0: // 初始化状态，没有选择图片点击
                _this.choosePictureAndUpload();
                break;
            case 1: // 选择了图片点击
                break;
            case 2: // 上传中点击
                // 取消上传
                if (_this.uploadRequest) {
                    _this.uploadRequest.abort();
                }
                _this.updateUI(0);
                break;
            case 3: // 上传成功点击
                _this.choosePictureAndUpload();
                break;
            case 4: // 上传失败点击
                _this.choosePictureAndUpload();
                break;
            default:
                break;
        }
    }

    choosePictureAndUpload() {
        let _this = this;

        choosePicture({
            multiple: false,
            onChange: function (files) {
                let file = files[0];
                if (!file) {
                    return
                }

                if (_this.coverURL) {
                    URL.revokeObjectURL(_this.coverURL);
                }

                if (_this.uploadRequest) {
                    _this.uploadRequest.abort();
                }

                // 选择了图片
                let url = URL.createObjectURL(file);
                _this.updateUI(1, url);

                // 开始上传
                _this.uploadRequest = uploadPicture({
                    file: file,
                    watermark: false,
                    progress: function (percent) {
                        _this.updateUI(2, percent);
                    },
                    success: function (data) {
                        try {
                            if (data.result.img_url) {
                                _this.updateUI(3, data);
                                // 封面图片发生改变通知
                                native.cell({
                                    __action: 'coverOnChange'
                                });
                            } else {
                                _this.updateUI(4, data);
                            }
                        } catch (err) {
                            _this.updateUI(4, err);
                        }
                    },
                    error: function (err) {
                        _this.updateUI(4, err);
                    }
                });
            }
        });
    }

}

let coverImageManager = new CoverImageManager();


////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/**表示链接弹出描述卡片
 * @param {string} elem  当前点击元素
 * @param {number} type 当前卡片类型
 * @param {string} jqLeft left
 * @param {string} jqTop  Top
 * @param {string} href  链接
 *  @param {string} content  内容
 */
function LinkBubble($domNode,type,href,content){
    let objRect =  $domNode[0].getBoundingClientRect();
    let winTop = $(window).scrollTop();
    let winLeft = $(window).scrollLeft();
    let jqLeft = objRect.x + (objRect.width/2) + winLeft -61;
    let jqTop = objRect.y + objRect.height + winTop+13;
    let tip = type == 1?'切换为卡片':'切换为链接';
    if(jqLeft<5){
        jqLeft = 5
    }
    if($('.LinkBubble-content').length>0){
      $('.LinkBubble-content').css({
        left: jqLeft+'px',
         top: jqTop+'px'
      }).show();
      $('.LinkBubble-content .transfer').html(`${tip}`)
      $('.LinkBubble-content').attr('data-content',content);
    $('.LinkBubble-content').attr('data-href',href);
    $('.LinkBubble-content').attr('data-type',type);
     }else{
      $('body').append(`<div class="LinkBubble-content menu" style="left: ${jqLeft}px; top:${jqTop}px;" data-type="${type}" data-content="${content}" data-href="${href}">
          <span class="Button edit"><i></i></span>
      <span  class="Button transfer" >${tip}</span>
  </div>`)
     }
  }
  //编辑器链接
$('body').on('click','.LinkBubble-content .edit',function(e){
    let href = $(this).parent().attr('data-href');
    let content = $(this).parent().attr('data-content');
    let type = $(this).parent().attr('data-type');
      let message = {
        title: "编辑链接",
        type:2
        };
        let data={
            href:href,
            text:content
        }
    let desc = prompt(JSON.stringify(message), JSON.stringify(data));
    if (desc == null) {
        return 
    }
        desc = JSON.parse(desc)
     if(type == 1){
      LinkDom.text(desc.text);
      LinkDom.attr('href',desc.href);
    }else{
      LinkDom.find('.link-card-content').text(desc.text);
      LinkDom.find('.link-card-href').text(geturl(desc.href));
      LinkDom.attr('href',desc.href);
      if(desc.href != href){
        getUrlAjax(desc.href,LinkDom)//请求链接卡片地址标题域名
      }
     
    }
    $('.LinkBubble-content').remove();
  })

  $('body').click(function(){
    // $('.LinkBubble-content').hide();
  })
  $('body').on('click','.LinkBubble-content',function(e){
    //阻止冒泡
    stopPropagation(e)
  })
 //卡片链接切换方式
$('body').on('click','.LinkBubble-content .transfer',function(e){
    let href = $(this).parent().attr('data-href');
    let content = $(this).parent().attr('data-content');
    let type = $(this).parent().attr('data-type');
    if(type==2){
        let children = LinkDom[0].nextSibling
        let parent = LinkDom[0].parentNode
        //控制换行
        if(children && children.children && children.children[0].tagName == 'BR'){
            $(LinkDom[0].nextSibling).remove();
        }else if (parent&& parent.tagName == 'P'&& parent.nextSibling.children[0].tagName == 'BR'){
            $(parent.nextSibling).remove();
        }
        LinkDom.replaceWith(`<a href="${href}">${content}</a>`)
      
    }else{

        let innerHTML = popUp(3,content,geturl(href),bgImg)
        let newnode = document.createElement("div"); 
        let br = document.createElement("br");   
        newnode.className = 'link-card';
        newnode.setAttribute('href',href)
        newnode.setAttribute('contenteditable',false)
        newnode.setAttribute('target','_blank')
        let oldnode = LinkDom[0];
        newnode.innerHTML = innerHTML;
        oldnode.replaceWith(newnode);
        $(newnode.parentNode).after('<br>')
        getUrlAjax(href,newnode);
    }
   
    $('.LinkBubble-content').remove();
})
/**工具栏自定义弹窗\卡片
 * @param {number} type 类型
 * @param {string} text 内容
 * @param {string} href  链接
 * * @param {string} img  图片地址
 */
function popUp(type, text, href, img) {
    let content = "";
    switch (type) {
      case 3:
        //判断是否为默认图,如果是,去掉默认图
        let background = img;
        if(background.indexOf("dgtle_img/wap/net_normal")!=-1){
            background = "";
        }
        content = `<span class="link-card-background" style=" background-image: url('${background}') !important;"></span>
        <div class="link-content">
                      <span class="link-card-img" style=" background-image: url('${img}') !important;"></span>
                      <div class="link-card-right">
                        <span class="link-card-content">${text}</span>
                    <span class="link-card-href">${href}</span>
                      </div>
                  </div>`;
                  break;
      default:
        break;
    }
    return content;
  }
//请求链接卡片地址标题域名
function getUrlAjax(url,elem){
    let apiUrl = `${API_link}?url=${encodeURIComponent(url)}`;
    let authorization = sessionStorage.getItem("authorization");
    $.ajax({
      url:apiUrl,
      // dataType:"jsonp",    //跨域json请求一定是jsonp
      // jsonp: "callbackparam",    //跨域请求的参数名，默认是callback
      headers: {
        Authorization: authorization
      },
    // headers: {
    //     Authorization: 'Bearer eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJodHRwczpcL1wvcmVsZWFzZS5hcGkuZGd0bGUuY29tIiwiYXVkIjoiaHR0cHM6XC9cL3JlbGVhc2UuYXBpLmRndGxlLmNvbSIsImlhdCI6MTU4NjQyODk3NCwibmJmIjoxNTg2NDI4OTc0LCJleHAiOjE1ODkwMjA5NzMsInJmdCI6MTU4OTAyMDk3MywianRpIjoiN3AxcWp3ZHoifQ._FL4zlE2NABAMkwBBBVmUrTSuNf-LhJm2VOrtFtLpqI'
    //   },
      async:true,
      beforeSend:function(){
      },
      complete:function(){
      },
      success:function(data){
        let http = url.split('//')[0]
        let host = url.split('//')[1].split('/')[0];
        $(elem).find('.link-card-content').text(data.result.title)
        $(elem).find('.link-card-href').text(http+'//'+host)
        $(elem).find('.link-card-img').css('background-image',`url(${data.result.img||bgImg})`)
        $(elem).find('.link-card-background').css('background-image',`url(${data.result.img})`)
        
    },
  });
  }
//阻止冒泡
function stopPropagation(e){
    if (e && e.stopPropagation) {
      e.stopPropagation();
    } else {
      window.event.cancelBubble = true;
    }
  }

////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////



let native = {
    cell: function (json) {
        // iOS
       if (window.webkit && window.webkit.messageHandlers.bridge.postMessage) {
           window.webkit.messageHandlers.bridge.postMessage(json);
       }
       // Android
       else {
            disposeAndroidJson(json);
       }
    }
};
